#include <stdio.h>
#include <math.h>
#define PI 3.141592
#define MAX 8

typedef struct fcomplex { double r, i; } FCOMPLEX;
FCOMPLEX Cadd(FCOMPLEX a, FCOMPLEX b);
FCOMPLEX Cmul(FCOMPLEX a, FCOMPLEX b);
FCOMPLEX Cdiv(FCOMPLEX a, FCOMPLEX b);
FCOMPLEX CMPLX(double r, double i);

void dft(double data[], FCOMPLEX F[], int N);
void idft(FCOMPLEX F[], FCOMPLEX f[], int N);

int main()
{
	double data[MAX] =
	{ 0.0, 2.0, 4.0, 2.0, 0.0, -2.0, -4.0, -2.0 };
	FCOMPLEX F[MAX];
	FCOMPLEX f[MAX];
	double spectrum[MAX];
	int x, u, N;

	N = MAX;
	/*  Ǫ ȯ  */
	dft(data, F, N);
	for (u = 0; u < MAX; u++)
	{
		spectrum[u] = sqrt(F[u].r * F[u].r + F[u].i * F[u].i);
		printf("freq %d :  spectrum = %f\n", u, spectrum[u]);
	}
	/*  Ǫ ȯ */
	idft(F, f, N);
	for (x = 0; x < MAX; x++)
	{
		printf("%d :  data = %f, f = %f + j%f\n", x, data[x], f[x].r, f[x].i);
	}
	return 0;
}

FCOMPLEX Cadd(FCOMPLEX a, FCOMPLEX b)
{
	FCOMPLEX c;
	c.r = a.r + b.r;
	c.i = a.i + b.i;
	return c;
}

FCOMPLEX Cmul(FCOMPLEX a, FCOMPLEX b)
{
	FCOMPLEX c;
	c.r = a.r * b.r - a.i * b.i;
	c.i = a.i * b.r + a.r * b.i;
	return c;
}

FCOMPLEX Cdiv(FCOMPLEX a, FCOMPLEX b)
{
	FCOMPLEX c;
	double r, den;
	if (fabs(b.r) >= fabs(b.i))
	{
		r = b.i / b.r;
		den = b.r + r * b.i;
		c.r = (a.r + r * a.i) / den;
		c.i = (a.i - r * a.r) / den;
	}
	else
	{
		r = b.r / b.i;
		den = b.i + r * b.r;
		c.r = (a.r * r + a.i) / den;
		c.i = (a.i * r - a.r) / den;
	}
	return c;
}

FCOMPLEX CMPLX(double r, double i)
{
	FCOMPLEX c;
	c.r = r;
	c.i = i;
	return c;
}

void dft(double data[], FCOMPLEX F[], int N)
{
	int u, x;
	double b, c;
	FCOMPLEX a, d, e;
	FCOMPLEX sum;

	for (u = 0; u < N; u++)
	{
		sum = CMPLX(0, 0);
		for (x = 0; x < N; x++)
		{
			a = CMPLX(data[x], 0.0);
			b = cos(2 * PI * u * x / N);
			c = sin(2 * PI * u * x / N);
			d = CMPLX(b, -c);
			e = Cmul(a, d);

			sum = Cadd(sum, e);
		}
		a = CMPLX((double) N, 0.0);
		sum = Cdiv(sum, a);
		F[u] = sum;
	}
}

void idft(FCOMPLEX F[], FCOMPLEX f[], int N)
{
	int u, x;
	FCOMPLEX a, d, e;
	double b, c;

	FCOMPLEX sum;

	for (u = 0; u < N; u++)
	{
		sum = CMPLX(0, 0);
		for (x = 0; x < N; x++)
		{
			a = F[x];
			b = cos(2 * PI * u * x / (double) N);
			c = sin(2 * PI * u * x / (double) N);
			d = CMPLX(b, c);

			e = Cmul(a, d);
			sum = Cadd(sum, e);
		}
		f[u] = sum;
	}
}
